/***************************************************************************\
*                                                                           *
* File: dbg_memt.c                                                          *
*                                                                           *
* Desc: Test driver for checking the debug version of the dynamic memory    *
*       management functions.                                               *
*                                                                           *
* Vers: 3.00    Jim Roberts                                                 *
*                                                                           *
*       1.) Bunch'a changes; still not comprehensive.                       *
*                                                                           *
* Refs: Mike Daconta, "C Pointers and Dynamic Memory Management",  Chapter  *
*       13, Memory Management Internals, QED Publishing Group, Wellesley,   *
*       MA, 1993.                                                           *
*                                                                           *
\***************************************************************************/

#include "stdlib.h"
#include "stdio.h"
#include "string.h"
#include "ctype.h"
#include "alloc.h"

#include "local.h"

static void test_pause ( void );

int main ( void  )
   {
   char *   str0,
        *   str1,
        *   str2,
        *   str3,
        *   str4,
        *   str5,
        *   str6;
   char *   never_malloced;
   char     biiig_string[] = "This is a string too long to be stuffed into st1.";

   open_log ();
   printf ( "Test 1: Request malloc memory of varying sizes.\n" );
   log_comment ( "Begin test 1." );
   str1 = strdup ( "This is sentence one." );
   str2 = (char *)malloc ( 400 );
   str3 = strdup ( "This is sentence three and rather lengthy and long and very long." );
   str4 = (char *)malloc ( 600 );
   str5 = (char *)calloc ( 401 );
   str6 = (char *)malloc ( 3000 );
   if ( !str1 || !str2 || !str3 || !str4 || !str5 || !str6 )
      {
      printf ( "Trouble: malloc failed.\n" );
      log_comment ( "Unexpected NULL from malloc()" );
      exit ( EXIT_FAILURE );
      } /* if */
   memory_map ();
   print_stats ();

   printf ( "Test 2: Free some memory.\n" );
   log_comment ( "Begin test 2." );
   free ( str2 );
   free ( str4 );
   free ( str6 );
   memory_map ();
   print_stats ();

   printf ( "Test 3: Test reallocate of blocks.\n" );
   log_comment ( "Begin test 3." );
   str1 = (char *)realloc ( str1, strlen ( biiig_string ) + 1 );
   if ( str1 != NULL )
      {
      strcpy ( str1, biiig_string );
      str1 = (char *)realloc ( str1, strlen ( str1 ) + 1 );
      } /* if */
   str3 = (char *)realloc ( str1, strlen ( str1 ) - 10 );
   free ( str1 );                      /* Should fail; realloc() should have
                                          freed str1 in the above line. */
   memory_map ();
   print_stats ();

   printf ( "Test 4: Test error checking.\n" );
   log_comment ( "Begin test 4." );
   log_comment ( "Expecting error on malloc ( -1 )." );
   str0 = (char *)malloc ( -1 );
   log_comment ( "Expecting error on calloc ( -1 )." );
   str0 = (char *)calloc ( -1 );
   log_comment ( "Expecting error on realloc ( char *, -1 )." );
   str0 = (char *)realloc ( str0, -1 );
   log_comment ( "Expecting error on realloc ( already free, 1000 )" );
   str0 = (char *)realloc ( str2, 1000 );
   *( str2 + 3 ) = '*';
   log_comment ( "Expecting error on free block hammered, and" );
   log_comment ( "expecting error on free ( NULL )." );
   free ( NULL );
   log_comment ( "Expecting error on free ( already freed )." );
   free ( str4 );
   never_malloced = str1 + 5;
   log_comment ( "Expecting error on free ( never alloced )." );
   free ( never_malloced );

   printf ( "Test 5: Free memory and coalesce adjacent blocks.\n" );
   log_comment ( "Begin test 5." );
   free ( str3 );
   memory_map ();
   print_stats ();

   printf ( "Test 6: Test memory_map.\n" );
   log_comment ( "Begin test 6." );
      test_pause ();

   printf ( "Test 7: Check the heap for corruption. Should be good.\n" );
   log_comment ( "Begin test 7." );
   if ( !check_heap () )
      printf ( "Heap is corrupted!\n" );
   else
      printf ( "Heap is okay!\n" );

   printf ( "Test 8: allie, allie in free...\n" );
   log_comment ( "Begin test 8." );
   log_comment ( "Expecting block not free errors." );
   allie_in_free ();
   memory_map ();

   printf ( "Test 9: Corrupt the heap!\n" );
   log_comment ( "Begin test 9." );
   str2 = strdup ( "this is string2." );
   str3 = strdup ( "this is string3." );
   str4 = strdup ( "this is string4." );
   log_comment ( "Corrupting heap by writing off end of block." );
   strcpy ( str2, biiig_string );
   if ( !check_heap () )
      printf ( "Heap is corrupted as expected.\n" );
   else
      printf ( "Heap is okay; which isn't okay!\n" );
   print_stats ();
   close_log ();
   return EXIT_SUCCESS;
   } /* main */

void test_pause ( void )
   {
   char *  sBuf  = "999";
   int     lines = 0;
   char ** ppSet = NULL;
   char ** ppRel = NULL;

   printf ( "Testing multi-page memory_map output; how many strings? " );
   fgets ( sBuf, strlen ( sBuf ), stdin );
   lines = atoi ( sBuf );
   memory_map ();
   if ( lines < 1 )
      {
      printf ( "\nCute; but I'm not falling for it.\n" );
      return;
      } /* if */
   else if ( lines < 20 )
      {
      printf ( "\nWe've already done little memory_maps; I'll try 50.\n" );
      lines = 50;
      }
   else if ( lines > 100 )
      printf ( "\nWow; you are a glutton for punishment.\n" );
   ppSet = (char **)malloc ( lines * ( sizeof(char *) + 1 ) );
   if ( ppSet != NULL )
      {
      ppRel = ppSet;
      while ( lines-- > 0 )
         {
         *ppSet++ = (char *)malloc ( ( random ( 100 ) + 1 ) * sizeof(char) );
         if ( *ppSet == NULL )
            goto Alloc_Error;
         } /* while */
      *ppSet = NULL;
      } /* if */
   else
      goto Alloc_Error;
   memory_map ();
   while ( *ppRel != NULL )
      free ( *ppRel++ );
   memory_map ();
   return;

   Alloc_Error:
      {
      printf ( "Whoops, can't allocate memory for this test." );
      return;
      } /* Alloc_Error */

   } /* test_pause */

/* End of File */